function [] = main_cl(u_max, Q, R, T, Tf, x_init, N)
clearvars -except u_max Q R T Tf x_init N
% close all
clc

import casadi.*

% N = 50; % number of control intervals
nx = 2;
nu = 1;

N_sim = Tf/(T/N); 

% Declare model variables
x = MX.sym('x',nx,1);
u = MX.sym('u',nu,1);

% x_init = [0; 1];
x0_bar = MX.sym('x0_bar', 2, 1);

% Model equations
xdot = dynamics(x, u);

% Formulate discrete time dynamics
% Fixed step Runge-Kutta 4 integrator
M = 1; % RK4 steps per interval
DT = T/N/M;
f = Function('f', {x, u}, {xdot});
X0 = MX.sym('X0', nx,1);
U = MX.sym('U',nu,1);
X = X0;

for j=1:M
    k1 = f(X, U);
    k2 = f(X + DT/2 * k1, U);
    k3 = f(X + DT/2 * k2, U);
    k4 = f(X + DT * k3, U);
    X=X+DT/6*(k1 +2*k2 +2*k3 +k4);
end

F = Function('F', {X0, U}, {X});

% Evaluate at a test point
Fk = F([0.2; 0.3],0.4);
disp(Fk)

% Start with an empty NLP
w={};
w0 = [];
lbw = [];
ubw = [];
J = 0;
g={};
lbg = [];
ubg = [];

X0 = MX.sym('X0', nx, 1);
w0 = [w0;  0.1; 0.1];
w = {w{:}, X0};
lbw = [lbw; -inf; -inf];
ubw = [ubw;  inf;  inf];

g = {g{:}, X0 - x0_bar};
lbg = [lbg; 0; 0];
ubg = [ubg; 0; 0];

% Formulate the NLP
Xk = x0_bar;
for k=0:N-1
    % New NLP variable for the control
    Uk = MX.sym(['U_' num2str(k)], nu, 1);
    w = {w{:}, Uk};
    w0 = [w0;  0.1];
    lbw = [lbw; -u_max];
    ubw = [ubw; u_max];

    % Integrate till the end of the interval
    Xk_end =  F(Xk, Uk);
    J = J+ 1/2*(Xk.'*Q*Xk + R*Uk^2);

    % New NLP variable for state at end of interval
    Xk = MX.sym(['X_' num2str(k+1)], nx, 1);
    w = {w{:}, Xk};
    lbw = [lbw; -inf; -inf];
    ubw = [ubw;  inf;  inf];
    w0 = [w0; 0.1; 0.1];

    % Add equality constraint
    g = {g{:}, Xk_end-Xk};
    lbg = [lbg; 0; 0];
    ubg = [ubg; 0; 0];
end

J = J + 1/2*Xk.'*Q*Xk;

g = vertcat(g{:});
w = vertcat(w{:});

p_in = x0_bar;
% Create an NLP solver
prob = struct('f', J, 'x', w, 'g', g, 'p', p_in);
solver = nlpsol('solver', 'ipopt', prob);

% Solve the NLP
sol = solver('x0', w0, 'lbg', lbg, 'ubg', ubg, 'lbx', lbw, 'ubx', ubw, 'p', x_init);
       
max_it = 100;
iters = zeros(length(w),max_it);
X_cl = zeros(2,N_sim);
U_cl = zeros(1,N_sim);
X_cl(:,1) = x_init;

% RTI solver
for i=1:N_sim    
    
    sol = solver('x0', w0,'lbx',lbw, 'ubx', ubw, 'lbg', lbg, 'ubg', ubg , 'p', [X_cl(:,i)]);
    sol = full(sol.x);
    iters(:,i) = sol;
    U_cl(i) = sol(3);
    X_cl(:,i+1) = sol(4:5);

end

% figure()
subplot(211)
hold all
plot([0:N_sim]*T/N, X_cl.','black--')
grid on
subplot(212)
hold all
stairs([0:N_sim-1]*T/N, U_cl,'black--')
grid on